/**
 * Ninjs Request
 * Request wrapper using request module
 */
'use strict'

const _ = require('ninjs-lodash')
const request = require('request')

exports = module.exports = request
_.assign(exports, {
	toFile: toFile,
	writeHead: writeHead,
	writeHeaders: writeHeaders,
	writeRequest: writeRequest
})

function toFile({ url='', dest='', pretty=false }, callback) {
	if (!url || !_.isString(url)) return _.fail('Invalid url', callback)
	if (!dest || !_.isString(dest)) return _.fail('Invalid dest', callback)

   let ex = _.path.ex(dest) || ''
	if (!ex) return _.fail('Invalid dest extension', callback)

  pretty = _.isBoolean(pretty) ? pretty : false

	request(url, function(err, res, body) {
		if(err) return _.fail(err, callback)
		// BREAKS OCCASIONALLY WITH INVALID MUP AND IS VERY INEFFICIENT SO USE SPARINGLY
		let result = pretty && ex === 'html' ? _.prettyHtml(body) : body
		_.outputFile(dest, result, callback)
	})
}

function writeHead({ url='', dest='' }, callback) {
	if (!url || !_.isString(url)) return _.fail('Invalid url', callback)
	if (!dest || !_.isString(dest)) return _.fail('Invalid dest', callback)

	return _.async.waterfall([
    _.async.constant(url), 
    request.head, 
    _.async.apply(_.outputJson, dest)
  ], callback)
}

function writeHeaders({ url='', dest='' }, callback) {
	if (!url || !_.isString(url)) return _.fail('Invalid url', callback)
	if (!dest || !_.isString(dest)) return _.fail('Invalid dest', callback)

	_.async.waterfall([
		_.async.constant(url),
		request.head,
		_.async.asyncify(function(result) { return _.get(result.toJSON(), 'headers') }),
		_.async.apply(_.outputJson, dest)
	], callback)
}

function writeRequest({ url='', dest='' }, callback) {
	if (!url || !_.isString(url)) return _.fail('Invalid url', callback)
	if (!dest || !_.isString(dest)) return _.fail('Invalid dest', callback)

	_.async.waterfall([
		_.async.constant(url),
		request.head,
		_.async.asyncify(function (result) { return _.get(result.toJSON(), 'request') }),
		_.async.apply(_.outputJson, dest)
	], callback)
}