var _ = require("lodash"),
	path = require("path"),
	os = require("os");

var APPCMD_ROOT = path.join("c:", "Windows", "System32", "inetsrv", "appcmd.exe"),
	FFMPEG_ROOT = path.join("c:", "FFMPEG", "bin");

var CHALK_INFO = {
	"modifiers": ["reset", "bold", "dim", "italic", "underline", "inverse", "hidden", "strikethrough"],
	"colors": ["black", "red", "green", "yellow", "blue", "magenta", "cyan", "white", "gray"],
	"bg_colors": ["bgBlack", "bgRed", "bgGreen", "bgYellow", "bgBlue", "bgMagenta", "bgCyan", "bgWhite"]
};

module.exports = {
// Debug

	"debug": {
		"options": {
			"enabled": true,
			"src": "",
			"dest": "",
			"cache": { "store": "log" },
			"pad": 15,
			"color": "white",
			"timestamp": { "pad": 9 },
			"style": "default", // default output format["style"]
			"padding": 25, // padEnd padding for first string arg
			"indent": "   " // JSON Object index from parent to child
		},

		// output formatting / middleware
		"format_map": {

			// log('nomap', "here", "is", "a", "really", "long", "messsage");
			// output => 'here is a really long messsage'
			"default": function() {
				var args = _.drop(_.toArray(arguments));
				return console.log.apply(console, args);
			},

			// log("msg", "here", "is", "a", "really", "long", "messsage");
			// output => 'here                     : is a really long messsage'
			"msg": function() {
				// args[0]: "type" arg used to map this middleware
				// args[1]: first real arg is padEnded
				var args = _.toArray(arguments),
					type = args.shift(),
					head = args.shift();
				if(!head || !_.isString(head)) return;

				head = _.padEnd(head, opt("padding"));
				console.log.apply(console, _.concat(["%s: %s", head], args));
			},

			// output => pretty json object string
			"jmsg": function() {
				var args = _.toArray(arguments),
					type = args.shift(),
					head = args.shift();
				if(!_.isPlainObject(head)) return console.log(head);
				console.log();
				console.log(JSON.stringify(head, null, opt("indent")));
			},

			// logs json as pretty printed stat object (NO BRACES, ETC)
			"jstats": function() {
				var args = _.toArray(arguments);
				_.each(args, function(arg) {
					if(!_.isObject(arg)) return;
					console.log(prettyjson.render(arg, opt("prettyjson")));
				});
			}
		},

		// output key -> color map
		"color_map": {

			// generic
			"timestamp": "bold.gray",
			"event": "bold.green",
			"error": "bold.red",
			"fail": "bold.red",
			"success": "bold.green",
			"done": "bold.green",

			// typed
			"monitor": "bold.cyan",
			"queue": "bold.yellow",
			"task": "bold.blue",
			"core": "bold.magenta",

			// custom
			"process": "bold.white",
			"goodbye": "bold.cyan"
		}
	},

// NPM

	"npm": {
		"packages": {
			"lodash": {
				"excludes": [
					//"templateSettings",
					//"after",
					"ary",
					//"assign",
					"assignIn",
					"assignInWith",
					"assignWith",
					//"at",
					//"before",
					//"bind",
					"bindAll",
					"bindKey",
					"castArray",
					//"chain",
					"chunk",
					//"compact",
					//"concat",
					"cond",
					"conforms",
					//"constant",
					"countBy",
					"create",
					"curry",
					"curryRight",
					//"debounce",
					//"defaults",
					//"defaultsDeep",
					"defer",
					//"delay",
					//"difference",
					"differenceBy",
					"differenceWith",
					//"drop",
					//"dropRight",
					"dropRightWhile",
					"dropWhile",
					"fill",
					//"filter",
					"flatMap",
					"flatMapDeep",
					"flatMapDepth",
					"flatten",
					"flattenDeep",
					"flattenDepth",
					"flip",
					"flow",
					"flowRight",
					"fromPairs",
					//"functions",
					//"functionsIn",
					//"groupBy",
					//"initial",
					//"intersection",
					"intersectionBy",
					"intersectionWith",
					"invert",
					"invertBy",
					//"invokeMap",
					"iteratee",
					//"keyBy",
					//"keys",
					//"keysIn",
					//"map",
					"mapKeys",
					"mapValues",
					//"matches",
					"matchesProperty",
					"memoize",
					//"merge",
					"mergeWith",
					//"method",
					//"methodOf",
					//"mixin",
					//"negate",
					//"nthArg",
					//"omit",
					"omitBy",
					//"once",
					//"orderBy",
					"over",
					"overArgs",
					"overEvery",
					"overSome",
					//"partial",
					//"partialRight",
					"partition",
					//"pick",
					"pickBy",
					//"property",
					//"propertyOf",
					"pull",
					"pullAll",
					"pullAllBy",
					"pullAllWith",
					"pullAt",
					"range",
					"rangeRight",
					"rearg",
					"reject",
					//"remove",
					//"rest",
					//"reverse",
					"sampleSize",
					//"set",
					"setWith",
					"shuffle",
					//"slice",
					//"sortBy",
					"sortedUniq",
					"sortedUniqBy",
					//"split",
					"spread",
					"tail",
					"take",
					"takeRight",
					"takeRightWhile",
					"takeWhile",
					//"tap",
					//"throttle",
					//"thru",
					//"toArray",
					"toPairs",
					"toPairsIn",
					//"toPath",
					//"toPlainObject",
					//"transform",
					"unary",
					//"union",
					"unionBy",
					"unionWith",
					//"uniq",
					"uniqBy",
					"uniqWith",
					//"unset",
					//"unzip",
					"unzipWith",
					//"update",
					"updateWith",
					//"values",
					//"valuesIn",
					//"without",
					//"words",
					//"wrap",
					"xor",
					//"xorBy",
					"xorWith",
					//"zip",
					//"zipObject",
					//"zipObjectDeep",
					//"zipWith",
					"entries",
					"entriesIn",
					//"extend",
					//"extendWith",
					//"add",
					//"attempt",
					"camelCase",
					"capitalize",
					"ceil",
					"clamp",
					//"clone",
					//"cloneDeep",
					//"cloneDeepWith",
					//"cloneWith",
					"conformsTo",
					"deburr",
					//"defaultTo",
					"divide",
					//"endsWith",
					//"eq",
					//"escape",
					//"escapeRegExp",
					"every",
					//"find",
					"findIndex",
					//"findKey",
					"findLast",
					"findLastIndex",
					"findLastKey",
					//"floor",
					//"forEach",
					"forEachRight",
					//"forIn",
					"forInRight",
					"forOwn",
					"forOwnRight",
					//"get",
					"gt",
					"gte",
					//"has",
					//"hasIn",
					//"head",
					//"identity",
					//"includes",
					//"indexOf",
					"inRange",
					//"invoke",
					//"isArguments",
					//"isArray",
					//"isArrayBuffer",
					//"isArrayLike",
					//"isArrayLikeObject",
					//"isBoolean",
					//"isBuffer",
					//"isDate",
					//"isElement",
					//"isEmpty",
					//"isEqual",
					//"isEqualWith",
					//"isError",
					//"isFinite",
					//"isFunction",
					//"isInteger",
					//"isLength",
					// "isMap",
					// "isMatch",
					// "isMatchWith",
					// "isNaN",
					// "isNative",
					// "isNil",
					// "isNull",
					// "isNumber",
					// "isObject",
					// "isObjectLike",
					// "isPlainObject",
					// "isRegExp",
					// "isSafeInteger",
					// "isSet",
					// "isString",
					// "isSymbol",
					// "isTypedArray",
					// "isUndefined",
					// "isWeakMap",
					// "isWeakSet",
					//"join",
					//"kebabCase",
					//"last",
					//"lastIndexOf",
					//"lowerCase",
					//"lowerFirst",
					//"lt",
					//"lte",
					//"max",
					"maxBy",
					"mean",
					"meanBy",
					//"min",
					"minBy",
					//"stubArray",
					//"stubFalse",
					//"stubObject",
					//"stubString",
					//"stubTrue",
					"multiply",
					//"nth",
					"noConflict",
					"noop",
					//"now",
					//"pad",
					//"padEnd",
					//"padStart",
					//"parseInt",
					//"random",
					//"reduce",
					"reduceRight",
					"repeat",
					//"replace",
					"result",
					"round",
					"runInContext",
					"sample",
					//"size",
					//"snakeCase",
					"some",
					"sortedIndex",
					"sortedIndexBy",
					"sortedIndexOf",
					"sortedLastIndex",
					"sortedLastIndexBy",
					"sortedLastIndexOf",
					//"startCase",
					//"startsWith",
					//"subtract",
					"sum",
					"sumBy",
					//"template",
					"times",
					//"toFinite",
					//"toInteger",
					//"toLength",
					// "toLower",
					// "toNumber",
					// "toSafeInteger",
					// "toString",
					// "toUpper",
					//"trim",
					//"trimEnd",
					//"trimStart",
					//"truncate",
					//"unescape",
					//"uniqueId",
					//"upperCase",
					//"upperFirst",
					//"each",
					"eachRight",
					//"first",
					//"VERSION"

				]
			},
			"prettyjson": {
				noColor: false, // disables colors
				// keysColor: 'rainbow', // 'rainbow', 'trap'
				// dashColor: 'magenta', // 'magenta'
				// stringColor: 'white' // 'white'
			},
			"jstr": {
				"wrap": 60, // line width
				"indent": "   ", // left indent space from parent to children
				"short": false, // bunch info together
				"sort": false, // alphabetically sort keys
				"aligned": true, // pad keys
				"decimals": -1, // how many trailing digits after .
				"arrayPadding": 1, // space after opening bracket and before closing bracket
				"objectPadding": 1, // space after opening brace and before closing brace
				"afterComma": 1, // space after a comma
				"beforeComma": 0, // space before a comma
				"afterColon": 1, // space after a colon
				"beforeColon": 1 // space before a colon
			}
		}
	},

// Paths

	"paths": {
		"cwd": process.cwd(), // process current working directory
		"entry": require.main.filename // main entry point file path (should be cwd unless started away from root)
	},

// IIS - Internet Information Services (Windows Server)

	"iis": {
		"root": "",
		"path": APPCMD_ROOT // Configure IIS
	},

// FFMPEG Video Processing

	"ffmpeg": {
		"path": FFMPEG_ROOT
	},

// Command Line

	"argv": { }, // command line argv converted to json obj (via minimist)

// Environment

	"env": {
		"env": "production", // NODE_ENV -> "local","development", "test", "staging","production"
		"port": 3000, // Default server port
		"cwd": process.cwd(), // process current working directory
		"entry": require.main.filename,
		"argv": process.argv, // command line argv converted to json obj (via jav)
		"eol": os.EOL, // platform end of line character
		"delimeter": path.delimeter,
		"vars": {
			"NODE_ENV" : "production", // NODE_ENV -> "local","development", "test", "staging","production"
			"DEBUG" : "*"
		},
		"map": {
			"local": {

			},
			"development": {

			},
			"production": {

			}
		}
	},

// Server

	"server": {
		"port": 3000 // Default server port
	},

// OS

	"os": {
		"eol": os.EOL // platform end of line character
	},

// Globals

	"globals": {},

// Package

	"package": {},

// Messages

	// log messages map
	"messages": {
		"exit": "goodbye" // process exit message
	},

// Errors

	// error message mappings
	"errors": {
		"unknown": "Unknown Error" // Unknown Error Message
	},

// Mime Types

	// mime type mappings
	"mime_types": {},

// Lodash Settings / Mixins

	// lodash config
	"_": {
		"mixins": ["*"],
		"omits": [

			//"templateSettings",
			//"after",
			"ary",
			//"assign",
			"assignIn",
			"assignInWith",
			"assignWith",
			//"at",
			"before",
			//"bind",
			//"bindAll",
			//"bindKey",
			"castArray",
			//"chain",
			"chunk",
			//"compact",
			//"concat",
			"cond",
			"conforms",
			//"constant",
			"countBy",
			"create",
			"curry",
			"curryRight",
			"debounce",
			"defaults",
			"defaultsDeep",
			"defer",
			"delay",
			"difference",
			"differenceBy",
			"differenceWith",
			"drop",
			"dropRight",
			"dropRightWhile",
			"dropWhile",
			"fill",
			"filter",
			"flatMap",
			"flatMapDeep",
			"flatMapDepth",
			"flatten",
			"flattenDeep",
			"flattenDepth",
			"flip",
			"flow",
			"flowRight",
			"fromPairs",
			"functions",
			"functionsIn",
			"groupBy",
			"initial",
			"intersection",
			"intersectionBy",
			"intersectionWith",
			"invert",
			"invertBy",
			"invokeMap",
			"iteratee",
			"keyBy",
			"keys",
			"keysIn",
			"map",
			"mapKeys",
			"mapValues",
			"matches",
			"matchesProperty",
			"memoize",
			"merge",
			"mergeWith",
			"method",
			"methodOf",
			"mixin",
			"negate",
			"nthArg",
			"omit",
			"omitBy",
			"once",
			"orderBy",
			"over",
			"overArgs",
			"overEvery",
			"overSome",
			"partial",
			"partialRight",
			"partition",
			"pick",
			"pickBy",
			"property",
			"propertyOf",
			"pull",
			"pullAll",
			"pullAllBy",
			"pullAllWith",
			"pullAt",
			"range",
			"rangeRight",
			"rearg",
			"reject",
			"remove",
			"rest",
			"reverse",
			"sampleSize",
			"set",
			"setWith",
			"shuffle",
			"slice",
			"sortBy",
			"sortedUniq",
			"sortedUniqBy",
			"split",
			"spread",
			"tail",
			"take",
			"takeRight",
			"takeRightWhile",
			"takeWhile",
			"tap",
			"throttle",
			"thru",
			"toArray",
			"toPairs",
			"toPairsIn",
			"toPath",
			"toPlainObject",
			"transform",
			"unary",
			"union",
			"unionBy",
			"unionWith",
			"uniq",
			"uniqBy",
			"uniqWith",
			"unset",
			"unzip",
			"unzipWith",
			"update",
			"updateWith",
			"values",
			"valuesIn",
			"without",
			"words",
			"wrap",
			"xor",
			"xorBy",
			"xorWith",
			"zip",
			"zipObject",
			"zipObjectDeep",
			"zipWith",
			"entries",
			"entriesIn",
			"extend",
			"extendWith",
			"add",
			"attempt",
			"camelCase",
			"capitalize",
			"ceil",
			"clamp",
			"clone",
			"cloneDeep",
			"cloneDeepWith",
			"cloneWith",
			"conformsTo",
			"deburr",
			"defaultTo",
			"divide",
			"endsWith",
			"eq",
			"escape",
			"escapeRegExp",
			"every",
			"find",
			"findIndex",
			"findKey",
			"findLast",
			"findLastIndex",
			"findLastKey",
			"floor",
			"forEach",
			"forEachRight",
			"forIn",
			"forInRight",
			"forOwn",
			"forOwnRight",
			"get",
			"gt",
			"gte",
			"has",
			"hasIn",
			"head",
			"identity",
			"includes",
			"indexOf",
			"inRange",
			"invoke",
			"isArguments",
			"isArray",
			"isArrayBuffer",
			"isArrayLike",
			"isArrayLikeObject",
			"isBoolean",
			"isBuffer",
			"isDate",
			"isElement",
			"isEmpty",
			"isEqual",
			"isEqualWith",
			"isError",
			"isFinite",
			"isFunction",
			"isInteger",
			"isLength",
			"isMap",
			"isMatch",
			"isMatchWith",
			"isNaN",
			"isNative",
			"isNil",
			"isNull",
			"isNumber",
			"isObject",
			"isObjectLike",
			"isPlainObject",
			"isRegExp",
			"isSafeInteger",
			"isSet",
			"isString",
			"isSymbol",
			"isTypedArray",
			"isUndefined",
			"isWeakMap",
			"isWeakSet",
			"join",
			"kebabCase",
			"last",
			"lastIndexOf",
			"lowerCase",
			"lowerFirst",
			"lt",
			"lte",
			"max",
			"maxBy",
			"mean",
			"meanBy",
			"min",
			"minBy",
			"stubArray",
			"stubFalse",
			"stubObject",
			"stubString",
			"stubTrue",
			"multiply",
			"nth",
			"noConflict",
			"noop",
			"now",
			"pad",
			"padEnd",
			"padStart",
			"parseInt",
			"random",
			"reduce",
			"reduceRight",
			"repeat",
			"replace",
			"result",
			"round",
			"runInContext",
			"sample",
			"size",
			"snakeCase",
			"some",
			"sortedIndex",
			"sortedIndexBy",
			"sortedIndexOf",
			"sortedLastIndex",
			"sortedLastIndexBy",
			"sortedLastIndexOf",
			"startCase",
			"startsWith",
			"subtract",
			"sum",
			"sumBy",
			"template",
			"times",
			"toFinite",
			"toInteger",
			"toLength",
			"toLower",
			"toNumber",
			"toSafeInteger",
			"toString",
			"toUpper",
			"trim",
			"trimEnd",
			"trimStart",
			"truncate",
			"unescape",
			"uniqueId",
			"upperCase",
			"upperFirst",
			"each",
			"eachRight",
			"first"
		]
	},


// Editor

	"editor": {
		"fontFamily": "Consolas, 'Courier New', monospace", // Controls the font family.
		"fontWeight": "normal", // Controls the font weight.
		"fontSize": 14, // Controls the font size in pixels.
		"lineHeight": 0, // Controls the line height. Use 0 to compute the lineHeight from the fontSize.
		"lineNumbers": true, // Controls visibility of line numbers
		"glyphMargin": false, // Controls visibility of the glyph margin
		"rulers": [0], // Columns at which to show vertical rulers

		// Characters that will be used as word separators when doing word related navigations or operations
		"wordSeparators": "`~!@#$%^&*()-=+[{]}\\|;:'\",.<>/?",

		// The number of spaces a tab is equal to. This setting is overriden based on the file contents when `detectIndentation` is on.
		"tabSize": 4,

		// Insert spaces when pressing Tab. This setting is overriden based on the file contents when `detectIndentation` is on.
		"insertSpaces": false,

		"detectIndentation": true, // When opening a file, `tabSize` and `insertSpaces` will be detected based on the file contents.
		"roundedSelection": true, // Controls if selections have rounded corners
		"scrollBeyondLastLine": true, // Controls if the editor will scroll beyond the last line

		// Controls after how many characters the editor will wrap to the next line.
		// Setting this to 0 turns on viewport width wrapping (word wrapping). Setting this to -1 forces the editor to never wrap.
		"wrappingColumn": 300,

		"wordWrap": false, // Controls if lines should wrap. The lines will wrap at min(wrappingColumn, viewportWidthInColumns).
		"wrappingIndent": "same", // Controls the indentation of wrapped lines. Can be one of 'none', 'same' or 'indent'.
		"mouseWheelScrollSensitivity": 1,// A multiplier to be used on the `deltaX` and `deltaY` of mouse wheel scroll events
		"quickSuggestions": true, // Controls if quick suggestions should show up or not while typing
		"quickSuggestionsDelay": 10, // Controls the delay in ms after which quick suggestions will show up
		"parameterHints": true, // Enables parameter hints
		"autoClosingBrackets": true, // Controls if the editor should automatically close brackets after opening them
		"formatOnType": false, // Controls if the editor should automatically format the line after typing
		"suggestOnTriggerCharacters": true, // Controls if suggestions should automatically show up when typing trigger characters

		// Controls if suggestions should be accepted 'Enter' - in addition to 'Tab'.
		// Helps to avoid ambiguity between inserting new lines or accepting suggestions.
		"acceptSuggestionOnEnter": true,

		// Controls whether snippets are shown with other suggestions and how they are sorted.
		"snippetSuggestions": "bottom",

		"wordBasedSuggestions": true, // Enable word based suggestions.

		// Insert snippets when their prefix matches. Works best when 'quickSuggestions' aren't enabled.
		"tabCompletion": false,

		"selectionHighlight": true, // Controls whether the editor should highlight similar matches to the selection

		// Controls the number of decorations that can show up at the same position in the overview ruler
		"overviewRulerLanes": 3,

		// Control the cursor animation style, possible values are 'blink', 'smooth', 'phase', 'expand' and 'solid'
		"cursorBlinking": "blink",

		"mouseWheelZoom": false, // Zoom the font of the editor when using mouse wheel and holding Ctrl
		"cursorStyle": "line", // Controls the cursor style, accepted values are 'block', 'line' and 'underline'
		"fontLigatures": false, // Enables font ligatures
		"hideCursorInOverviewRuler": false, // Controls if the cursor should be hidden in the overview ruler.
		"renderWhitespace": false, // Controls whether the editor should render whitespace characters
		"renderControlCharacters": false, // Controls whether the editor should render control characters
		"renderIndentGuides": false, // Controls whether the editor should render indent guides
		"renderLineHighlight": true, // Controls whether the editor should render the current line highlight
		"codeLens": true, // Controls if the editor shows code lenses
		"folding": true, // Controls whether the editor has code folding enabled
		"useTabStops": true, // Inserting and deleting whitespace follows tab stops
		"trimAutoWhitespace": true, // Remove trailing auto inserted whitespace

		// Keep peek editors open even when double clicking their content or when hitting Escape.
		"stablePeek": false
	},

// Diff Editor

	"diffEditor": {
		// Controls if the diff editor shows the diff side by side or inline
		"renderSideBySide": true,
		// Controls if the diff editor shows changes in leading or trailing whitespace as diffs
		"ignoreTrimWhitespace": true
	},

// Emmet

	"emmet": {
		"triggerExpansionOnTab": true, // When enabled, emmet abbreviations are expanded when pressing TAB.
		"preferences": {}, // Preferences used to modify behavior of some actions and resolvers of Emmet.
		"syntaxProfiles": {}, // Define profile for specified syntax or use your own profile with specific rules.
		"excludeLanguages": [] // An array of languages where emmet abbreviations should not be expanded.
	},

// Workbench

	"workbench": {
		"editor": {
			"showTabs": true, // Controls if opened editors should show in tabs or not.

			// Controls if opened editors show as preview.
			// Preview editors are reused until they are kept (e.g. via double click or editing).
			"enablePreview": true,

			 // Controls if opened editors from Quick Open show as preview.
			// Preview editors are reused until they are kept (e.g. via double click or editing).
			"enablePreviewFromQuickOpen": true,

			// Controls where editors open.
			// Select 'left' or 'right' to open editors to the left or right of the current active one.
    		// Select 'first' or 'last' to open editors independently from the currently active one.
			"openPositioning": "right"
		},
		"quickOpen" : {
			"closeOnFocusLost": true // Controls if Quick Open should close automatically once it loses focus.
		},
		"settings" : {
			"openDefaultSettings": true // Controls if opening settings also opens an editor showing all default settings.
		}
	},

// Window

	"window": {
		"openFilesInNewWindow": true, // When enabled, will open files in a new window instead of reusing an existing instance.

		// Controls how folders are being reopened after a restart.
		// 'none' - never reopen a folder
		// 'one'  - reopen the last folder you worked on
		// 'all'  - reopen all folders of your last session
		"reopenFolders": "none",

		"restoreFullscreen": false, // should window restore to full screen mode if it was exited in full screen mode.

		// Adjust the zoom level of the window.
		// 0 = 100%, 2.5 = 150%, 2 = 140%, 1 = 120%, -1 = 80%, -2 = 60%, -2.5 = 50%
		"zoomLevel": 0
	},

// Files

	"files": {
		// Configure glob patterns for excluding files and folders.
		"exclude": { "**/.git": true, "**/.svn": true, "**/.hg": true, "**/.DS_Store": true },

		// Configure file associations to languages (e.g. "*.extension": "html").
		// These have precedence over the default associations of the languages installed.
		"associations": {},

		"encoding": "utf8", // The default character set encoding to use when reading and writing files.
		"eol": "\n", // The default end of line character.
		"trimTrailingWhitespace": true, // When enabled, will trim trailing whitespace when you save a file.

		// Controls auto save of dirty files.
		// "off", "afterDelay", "onFocusChange" (editor loses focus), "onWindowChange" (window loses focus)
		// If set to "afterDelay", you can configure the delay in "files.autoSaveDelay".
		"autoSave": "afterDelay",

		// Controls the delay in ms after which a dirty file is saved automatically.
		// Only applies when "files.autoSave" is set to "afterDelay"
		"autoSaveDelay": 1000,

		// Configure glob patterns of file paths to exclude from file watching.
		// Changing this setting requires a restart. When you experience Code consuming lots of cpu time on startup,
		// you can exclude large folders to reduce the initial load.
		"watcherExclude": { "**/.git/objects/**": true }
	},

// File Explorer

	"explorer" : {
		"openEditors": {
			"visible": 9, // max # of editors shown in the Open Editors pane (0 hides it)
			"dynamicHeight": true // Open Editors pane dynamic item height change
		},
		"autoReveal": true, // automatically reveal files when opening them
		"enableDragAndDrop": true // allow drag and drop to move files and folders
	},

// Search

	"search": {
		// Search glob pattern excludes
		// ** Inherits files.exclude setting
		"exclude": { "**/node_modules": true, "**/bower_components": true },
		"quickOpen": {
			"includeSymbols": false // include results from a global symbol search in file results
		}
	},

// Git

	"git": {
		"enabled": true, // Enabled?
		"path": null, // Path to the git exe
		"autorefresh": true, // Auto-refreshing?
		"autofetch": true, // Auto-fetching?
		"enableLongCommitWarning": true, // Warn when commit message is too long?
		"allowLargeRepositories": false, // Code managed large repos?
		"confirmSync": false, // Confirm before sync?
		"countBadge": "all" // Git badge counter
	},

// HTTP

	"http": {
		"proxy": "", // Proxy setting (defaults to HTTP_PROXY / HTTPS_PROXY env vars)
		"proxyStrictSSL": true, // Verify proxy server certificate against list of supplied CAs?
		"proxyAuthorization": null // 'Proxy-Authorization' header value for every network request
	},

// Update

	"update": {
		 "channel": "default" // Automatic updates? (Requires Restart)
	},

// CSS

	// Controls CSS validation and problem severities.
	"css": {
		"validate": true, // Validations Enabled?
		"lint": {
			"compatibleVendorPrefixes": "ignore", // include all other vendor-specific properties
			"vendorPrefix": "warning", // include the standard property for vendor-specific properties
			"duplicateProperties": "ignore", // Do not use duplicate style definitions
			"emptyRules": "warning", // Do not use empty rulesets
			"importStatement": "ignore", // Import statements do not load in parallel
			"boxModel": "ignore", // Do not use width or height when using padding or border
			"universalSelector": "ignore", // The universal selector (*) is known to be slow
			"zeroUnits": "ignore", // No unit for zero needed
			"fontFaceProperties": "warning", // @font-face rule must define 'src' and 'font-family' properties
			"hexColorLength": "error", // Hex colors must consist of three or six hex numbers
			"argumentsInColorFunction": "error", // Invalid number of parameters
			"unknownProperties": "warning", // Unknown property
			"ieHack": "ignore", // IE hacks are only necessary when supporting IE7 and older
			"unknownVendorSpecificProperties": "ignore", // Unknown vendor specific property

			// Property is ignored due to the display
			// E.g. with 'display: inline', the width, height, margin-top, margin-bottom, and float properties have no effect
			"propertyIgnoredDueToDisplay": "warning",

			"important": "ignore", // Avoid using !important
			"float": "ignore", // Avoid using 'float'
			"idSelector": "ignore" // Selectors should not contain IDs
		}
	},

// SCSS (Sass)

	// Controls SCSS validation and problem severities.
	"scss": {
    	"validate": true, // Validations Enabled?
		"lint": {
			"compatibleVendorPrefixes": "ignore", // include all other vendor-specific properties
			"vendorPrefix": "warning", // include the standard property with vendor-specific properties
			"duplicateProperties": "ignore", // Do not use duplicate style definitions
			"emptyRules": "warning", // Do not use empty rulesets
			"importStatement": "ignore", // Import statements do not load in parallel
			"boxModel": "ignore", // Do not use width or height when using padding or border
			"universalSelector": "ignore", // The universal selector (*) is known to be slow
			"zeroUnits": "ignore", // No unit for zero needed
			"fontFaceProperties": "warning", // @font-face rule must define 'src' and 'font-family' properties
			"hexColorLength": "error", // Hex colors must consist of three or six hex numbers
			"argumentsInColorFunction": "error", // Invalid number of parameters
			"unknownProperties": "warning", // Unknown property
			"ieHack": "ignore", // IE hacks are only necessary when supporting IE7 and older
			"unknownVendorSpecificProperties": "ignore", // Unknown vendor specific property.

			// Property is ignored due to the display.
			// E.g. with 'display: inline', the width, height, margin-top, margin-bottom, and float properties have no effect
			"propertyIgnoredDueToDisplay": "warning",

			"important": "ignore", // Avoid using !important
			"float": "ignore", // Avoid using 'float'
			"idSelector": "ignore" // Selectors should not contain IDs
		}
	},

// LESS

    // Controls LESS validation and problem severities.
	"less" : {
		"validate": true, // Validations Enabled?
		"lint": {
			"compatibleVendorPrefixes": "ignore", // include all other vendor-specific properties
			"vendorPrefix": "warning", // include the standard property with vendor-specific properties
			"duplicateProperties": "ignore", // Do not use duplicate style definitions
			"emptyRules": "warning", // Do not use empty rulesets
			"importStatement": "ignore", // Import statements do not load in parallel
			"boxModel": "ignore", // Do not use width or height when using padding or border
			"universalSelector": "ignore", // The universal selector (*) is known to be slow
			"zeroUnits": "ignore", // No unit for zero needed
			"fontFaceProperties": "warning", // @font-face rule must define 'src' and 'font-family' properties
			"hexColorLength": "error", // Hex colors must consist of three or six hex numbers
			"argumentsInColorFunction": "error", // Invalid number of parameters
			"unknownProperties": "warning", // Unknown property
			"ieHack": "ignore", // IE hacks are only necessary when supporting IE7 and older
			"unknownVendorSpecificProperties": "ignore", // Unknown vendor specific property.

			// Property is ignored due to the display.
			// E.g. with 'display: inline', the width, height, margin-top, margin-bottom, and float properties have no effect
			"propertyIgnoredDueToDisplay": "warning",

			"important": "ignore", // Avoid using !important
			"float": "ignore", // Avoid using 'float'
			"idSelector": "ignore" // Selectors should not contain IDs
		}
	},

// HTML

	"html": {
		"format": {
			"wrapLineLength": 120, // Max amount of characters per line (0 = disable)

			// List of tags, comma separated, that shouldn't be reformatted
			// 'null' defaults to all tags listed at https://www.w3.org/TR/html5/dom.html#phrasing-content
			"unformatted": "a, abbr, acronym, b, bdo, big, br, button, cite, code, dfn, em, i, img, input, kbd, label, map, object, q, samp, script, select, small, span, strong, sub, sup, textarea, tt, var",

			"indentInnerHtml": false, // Indent <head> and <body> sections
			"preserveNewLines": true, // Whether existing line breaks before elements should be preserved. Only works before elements, not inside tags or for text
			"maxPreserveNewLines": null, // Maximum number of line breaks to be preserved in one chunk. Use 'null' for unlimited
			"indentHandlebars": false, // Format and indent {{#foo}} and {{/foo}}
			"endWithNewline": false, // End with a newline

			// List of tags, comma separated, that should have an extra newline before them. 'null' defaults to "head, body, /html"
			"extraLiners": "head, body, /html"
		},
		"suggest": {
			"angular1": true, // Configures if the built-in HTML language support suggests Angular V1 tags and properties
			"ionic": true, // Configures if the built-in HTML language support suggests Ionic tags, properties and values
			"html5": true // Configures if the built-in HTML language support suggests HTML5 tags, properties and values
		}
	},

// JSON

	// Array Formatting
	// [<arrayPadding>"first",<afterComma>"second"<beforeComma>,<afterComma>"last"<arrayPadding>]

	// Plain Object Formatting
	// {<objectPadding>"first"<beforeColon>:<afterColon>"item"<beforeComma>,<afterComma>"last"<beforeColon>:<afterColon>"item"<objectPadding>}

	"json": {
		"schemas": [], // Associate schemas to JSON files in the current project
		"format": {
			"wrap": 60, // line width
			"indent": "   ", // left indent space from parent to children
			"short": false, // bunch info together
			"sort": false, // alphabetically sort keys
			"aligned": true, // pad keys
			"decimals": -1, // how many trailing digits after .
			"arrayPadding": 1, // space after opening bracket and before closing bracket
			"objectPadding": 1, // space after opening brace and before closing brace
			"afterComma": 1, // space after a comma
			"beforeComma": 0, // space before a comma
			"afterColon": 1, // space after a colon
			"beforeColon": 1 // space before a colon
		}
	},

// Markdown

    // A list of URLs or local paths to CSS style sheets to use from the markdown preview.
	// Relative paths are interpreted relative to the folder open in the explorer.
	// If there is no open folder, they are interpreted relative to the location of the markdown file.
	// All '\' need to be written as '\\'.
    "markdown": {
		"styles": []
	},

// PHP

	"php": {
		"validate": {
			"enable": true, // Validation Enabled?
			"executablePath": null, // Points to the php executable
			"run": "onSave" // Whether the linter is run on save or on type
		}
	},

// TypeScript

	"typescript": {
		"tsdk": null, // Specifies the folder path containing the tsserver and lib*.d.ts files to use.
		"tsserver": {
			"trace": "off", // Enables tracing of messages send to the TS server

			// Enables experimental auto build.
			// Requires 1.9 dev or 2.x tsserver version and a restart of VS Code after changing it.
			"experimentalAutoBuild": false
		},
		"useCodeSnippetsOnMethodSuggest": false, // Complete functions with their parameter signature.
		"validate": {
			"enable": true // Validation Enabled?
		},
		"format": {
			"insertSpaceAfterCommaDelimiter": true, // Defines space handling after a comma delimiter
			"insertSpaceAfterSemicolonInForStatements": true, //  Defines space handling after a semicolon in a for statement
			"insertSpaceBeforeAndAfterBinaryOperators": true, // Defines space handling after a binary operator
			"insertSpaceAfterKeywordsInControlFlowStatements": true, // Defines space handling after keywords in control flow statement
			"insertSpaceAfterFunctionKeywordForAnonymousFunctions": true, // Defines space handling after function keyword for anonymous functions
			"insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis": false, // Defines space handling after opening and before closing non empty parenthesis
			"insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets": false, // Defines space handling after opening and before closing non empty brackets
			"placeOpenBraceOnNewLineForFunctions": false, // Defines whether an open brace is put onto a new line for functions or not
			"placeOpenBraceOnNewLineForControlBlocks": false // Defines whether an open brace is put onto a new line for control blocks or not
		}
	},

// JavaScript

	"javascript": {
		"validate": {
			"enable": true // Validation Enabled?
		},
		"format": {
			"insertSpaceAfterCommaDelimiter": true, // Defines space handling after a comma delimiter
			"insertSpaceAfterSemicolonInForStatements": true, //  Defines space handling after a semicolon in a for statement
			"insertSpaceBeforeAndAfterBinaryOperators": true, // Defines space handling after a binary operator
			"insertSpaceAfterKeywordsInControlFlowStatements": true, // Defines space handling after keywords in control flow statement
			"insertSpaceAfterFunctionKeywordForAnonymousFunctions": true, // Defines space handling after function keyword for anonymous functions
			"insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis": false, // Defines space handling after opening and before closing non empty parenthesis
			"insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets": false, // Defines space handling after opening and before closing non empty brackets
			"placeOpenBraceOnNewLineForFunctions": false, // Defines whether an open brace is put onto a new line for functions or not
			"placeOpenBraceOnNewLineForControlBlocks": false // Defines whether an open brace is put onto a new line for control blocks or not
		}
	},

// Extensions

    "extensions": {
		"autoUpdate": false // Automatically update extensions
	},

// External Terminal

	"terminal": {

		// External Terminal
		"external": {
			"linuxExec": "xterm", // Customizes which terminal to run on Linux.
			"osxExec": "Terminal.app", // Customizes which terminal application to run on OS X.
			"windowsExec": "%COMSPEC%" // Customizes which terminal to run on Windows.
		},

		// Integrated Terminal
		"integrated": {

			"shell": {
				"linux": "sh", // The path of the shell that the terminal uses on Linux.
				"osx": "sh", // The path of the shell that the terminal uses on OS X.

				// The path of the shell that the terminal uses on Windows.
				// When using shells shipped with Windows (cmd, PowerShell or Bash on Ubuntu),
				// prefer C:\Windows\sysnative over C:\Windows\System32 to use the 64-bit versions.
				"windows": "C:\\Windows\\System32\\cmd.exe"
			},

			"shellArgs": {
				"linux": [], // The command line arguments to use when on the Linux terminal.
				"osx": [] // The command line arguments to use when on the OS X terminal.
			},

			"fontFamily": "", // Controls the font family of the terminal, this defaults to editor.fontFamily's value.
			"fontLigatures": false, // Controls whether font ligatures are enabled in the terminal.
			"fontSize": 0, // Controls the font size in pixels of the terminal, this defaults to editor.fontSize's value.
			"lineHeight": 1.2, // Controls the line height of the terminal, this number is multipled by the terminal font size to get the actual line-height in pixels.
			"cursorBlinking": true, // Controls whether the terminal cursor blinks.
			"setLocaleVariables": false, // Controls whether locale variables are set at startup of the terminal, this defaults to true on OS X, false on other platforms.

			// A set of command IDs whose keybindings will not be sent to the shell and instead always be handled by Code.
			// This allows the use of keybindings that would normally be consumed by the shell to act
			// the same as when the terminal is not focused
			// ie. ctrl+p to launch Quick Open.
			"commandsToSkipShell": [
				"editor.action.toggleTabFocusMode",
				"workbench.action.quickOpen",
				"workbench.action.showCommands",
				"workbench.action.terminal.copySelection",
				"workbench.action.terminal.focus",
				"workbench.action.terminal.focusNext",
				"workbench.action.terminal.focusPrevious",
				"workbench.action.terminal.kill",
				"workbench.action.terminal.new",
				"workbench.action.terminal.paste",
				"workbench.action.terminal.runSelectedText",
				"workbench.action.terminal.scrollDown",
				"workbench.action.terminal.scrollUp",
				"workbench.action.terminal.toggleTerminal"
			]
		},

		// A Better Way to Configure (CUSTOM FROM VSCODE settings.json)
		"linux": {
			"exe": "xterm", // Customizes which terminal to run on Linux.
			"shell": "sh", // The path of the shell that the terminal uses on Linux.
			"shellArgs": [] // The command line arguments to use when on the Linux terminal.
		},
		"osx": {
			"exe": "Terminal.app", // Customizes which terminal application to run on OS X.
			"shell": "sh", // The path of the shell that the terminal uses on OS X.
			"shellArgs": [] // The command line arguments to use when on the OS X terminal.
		},
		"windows": {
			"exe": "%COMSPEC%", // Customizes which terminal to run on Windows.

			// The path of the shell that the terminal uses on Windows.
			// When using shells shipped with Windows (cmd, PowerShell or Bash on Ubuntu),
			// prefer C:\Windows\sysnative over C:\Windows\System32 to use the 64-bit versions.
			"shell": "C:\\Windows\\System32\\cmd.exe",

			"shellArgs": [] // The command line arguments to use when on the Windows terminal.
		}
	},

// Problems View

	"problems": {
		"autoReveal": true // Controls if Problems view should automatically reveal files when opening them
	},

// Telemetry

	"telemetry": {
		"enableTelemetry": false, // Enable usage data and errors to be sent to Microsoft.
		"enableCrashReporter": false // Enable crash reports to be sent to Microsoft (Requires Restart)
	},

	// Auto file generation ... coming soon
	"gen": {
		"options": {
			"enabled": false
		},
		"map": {
			"dev-module": function() {

			}
		}
	}


};