/**
 * Element
 * Common Html Element Wrappers
 */

var _ = require("lodash"),
	async = require("async"),
	cheerio = require("cheerio");

/**
 * Globals
 */

// var imageBasePath = _appConfig ? _appConfig.get('image-base-path') : '',
//     img = imageBasePath ? imageBasePath + img : img;

// Shortcuts
var _onError = 'onerror="if(typeof(window[\'<%= errorCb %>\']) === \'function\') { window[\'<%= errorCb %>\'](this); }"',
	_cn = 'class="<%= cn %>"',
	_base = _cn + ' title="<%= title %>" alt="<%= alt %>"',
	_href = 'href="<%= src %>"',
	_target = 'target="<%= target %>"',
	_src = 'src="<%= src %>"',
	_wmode = 'wmode="<%= wmode %>"',
	_type = 'type="<%= type %>"',
	_fs = 'allowfullscreen="<%= fullscreen %>"',
	_sa = 'allowscriptaccess="<%= scriptaccess %>"',
	_ytsrc = 'src="https://i1.ytimg.com/vi/<%= id %>/<%= size %>.jpg"',
	_ytvidsrc = 'src="http://www.youtube.com/embed/<%= id %>' +
		'?enablejsapi=<%= enablejsapi %>&rel=<%= rel %>&autoplay=<%= autoplay %>' +
		'&wmode=<%= wmode %>&fs=<%= fs %>&showinfo=<%= showinfo %>&controls=<%= controls %>"',
	_fbsrc = 'src="https://graph.facebook.com/<%= id %>/picture?type=<%= size %>"',
	_dim = 'width="<%= w %>" height="<%= h %>"',
	_dataCoords = 'data-x="<%= x %>" data-y="<%= y %>"',
	_dataDim = 'data-w="<%= w %>" data-h="<%= h %>"';

//Templates
var _anchor = '<a ' + [_href, _base, _target].join(' ') + '><%= title %></a>',
	_image = '<img ' + [_src, _base, _dataDim, _dataCoords, _onError].join(' ') + ' />',
	_ytimage = '<img ' + [_ytsrc, _base, _dataDim, _dataCoords, _onError].join(' ') + ' />',
	_fbpicture = '<img ' + [_fbsrc, _base, _dataDim, _dataCoords, _onError].join(' ') + ' />',
	_iframe = '<iframe ' + [_cn, _dim, _src].join(' ') + ' frameborder="0" allowfullscreen="true" ></iframe>',
	_ytiframe = '<iframe ' + [_cn, _dim, _ytvidsrc].join(' ') + ' frameborder="0" allowfullscreen="true" ></iframe>',
	_embed = '<embed ' + [_cn, _dim, _src, _wmode, _type, _fs, _sa].join(' ') + ' />',
	_video = '<div ' + _cn + '>' +
		'<video <%= controls ? \'controls="\' + controls + \'"\' : \'\' %>" poster="<%= image %>" ' + _dim +
		' <%= autoplay ? \'autoplay="\' + autoplay + \'"\' : \'\' %>" preload="none" >' +
			'<source src="<%= video %>" type="<%= type %>" autoplay="<%= autoplay %>" autostart="<%= autoplay %>" />' +
			//'<object type="application/x-shockwave-flash" data="<%= flash %>" ' + _dim + '>' +
			//	'<param name="movie" value="<%= flash %>" />' +
			//	'<param name="allowFullScreen" value="<%= fullscreen %>" />' +
			//	'<param name="wmode" value="<%= wmode %>" />' +
			//	'<param name="flashVars" value="controls=true&amp;poster=<%= encodeURIComponent(image) %>&amp;src=<%= encodeURIComponent(video) %>" />' +
			//	'<img alt="<%= title %>" src="<%= image %>" ' + _dim + ' title="<%= errText %>" />' +
			//'</object>' +
		'</video>' +
		//'<p><strong><%= dlPrefix %></strong><a href="<%= video %>"><%= dlText %></a></p>' +
		'</div>';

var ELEM_DEFAULTS = {
	id: "",
	cn: "",
	type: "",
	wmode: "transparent",
	src: "",
	w: 0,
	h: 0,
	x: 0,
	y: 0,
	title: "",
	alt: "",
	size: "",
	errorCb: "__snImgError"
};

exports = module.exports = _.assign({}, cheerio, {
	anchor: anchor,
	image: image,
	ytimage: ytimage,
	fbpicture: fbpicture,
	video: video,
	iframe: iframe,
	ytiframe: ytiframe,
	embed: embed
});


//Anchors
function anchor(params) {
	params = acopy(ELEM_DEFAULTS, {
		cn: "sn-anchor",
		target: "_blank"
	}, params);
	return params.src ? _.template(_anchor)(params) : '';
}

//Image
function image(params) {
	params = acopy(ELEM_DEFAULTS, {
		cn: "sn-img"
	}, params);
	return params.src ? _.template(_image)(params) : '';
}

function ytimage(params) {
	params = acopy(ELEM_DEFAULTS, {
		cn: "sn-img sn-ytimg",
		size: "mqdefault"
	}, params);
	return params.id ? _.template(_ytimage)(params) : '';
}

function fbpicture(params) {
	params = acopy(ELEM_DEFAULTS, {
		cn: "sn-img sn-fbpic",
		size: "large"
	}, params);
	return params.id ? _.template(_fbpicture)(params) : '';
}

//Video
function video(params) {
	params = acopy(ELEM_DEFAULTS, {
		cn: "video sn-video",
		controls: "controls",
		video: "",
		image: "",
		autoplay: false,
		type: "video/mp4",
		flash: "http://flashfox.googlecode.com/svn/trunk/flashfox.swf",
		wmode: "transparent",
		fullscreen: true,
		errText: "No video playback capabilities, please download the video below",
		dlPrefix: "Download video:",
		dlText: "MP4 format"
	}, params);
	return params.video ? _.template(_video)(params) : '';
}

//IFrame
function iframe(params) {
	params = acopy(ELEM_DEFAULTS, {
		cn: "sn-iframe",
		w: '100%',
		h: '100%'
	}, params);
	return params.src ? _.template(_iframe)(params) : '';
}

//YouTube
function ytiframe(params) {
	params = acopy(ELEM_DEFAULTS, {
		cn: "sn-iframe sn-ytiframe",
		rel: 0,
		autoplay: 0,
		wmode: "transparent",
		fs: 1,
		enablejsapi: 1,
		controls: 1,
		showinfo: 0,
		w: '100%',
		h: '100%'
	}, params);
	return params.id ? _.template(_ytiframe)(params) : '';
}

//Embed
function embed(params) {
	params = acopy(ELEM_DEFAULTS, {
		wmode: "transparent",
		w: '100%',
		h: '100%',
		fullscreen: true,
		scriptaccess: true,
		type: "application/x-shockwave-flash"
	}, params);
	return params.src ? _.template(_embed)(params) : '';
}