/**
 * NINJS-CORE OAuth
 * Standard HTTP Authentication Method Wrappers
 */

var _ = require("lodash"),
	auth = require("oauth");

	/**
 * Exports
 */

exports = module.exports = auth;

_.assign(exports, {
	basic: basic,
	bear: bear,
	oauth: oauth,
	oauth2: oauth2,
	echo: echo
});

function basic(headers, username, password) {
	return !_.isString(username) || !_.isString(password) ? headers : _.assign({}, headers, {
		"Authorization": "BASIC " + new Buffer(username + ":" + password).toString("base64")
	});
}

function bear(headers, token) {
	return !_.isString(token) ? headers : _.assign({}, headers, {
		"Authorization": "BEARER " + token
	});
}

// "request_token_endpoint": "https://api.twitter.com/oauth/request_token",
// "access_token_endpoint": "https://api.twitter.com/oauth/access_token",
// "consumer_key": "3vfN5NiaVFAtn9y82rb1o0aD6",
// "consumer_secret": "ajndIpZeJh7yQMM75TBS9nU1ROeuD8VApA8ZIpm0xLUHwqO9HU",
// "oauth_version": "1.0A",
// "callback": "https://ext.snapptop.com/twitter/callback/",
// "sig_method": "HMAC-SHA1"

// OAuth.OAuth(requestUrl, accessUrl, consumerKey, consumerSecret, version, authorize_callback, signatureMethod, nonceSize, customHeaders)
function oauth(options) {
	return new auth.OAuth(
		_.get(options, "request_token_endpoint", ""),
		_.get(options, "access_token_endpoint", ""),
		_.get(options, "consumer_key", ""),
		_.get(options, "consumer_secret", ""),
		_.get(options, "oauth_version", "1.0A"),
		_.get(options, "callback", ""),
		_.get(options, "sig_method", "HMAC-SHA1"));
}

// "url": "https://api.twitter.com/",
// "bearer_token_endpoint": "https://api.twitter.com/oauth2/token",
// "consumer_key": "3vfN5NiaVFAtn9y82rb1o0aD6",
// "consumer_secret": "ajndIpZeJh7yQMM75TBS9nU1ROeuD8VApA8ZIpm0xLUHwqO9HU",
//
// OAuth.OAuth2(clientId, clientSecret, baseSite, authorizePath, accessTokenPath, customHeaders)
function oauth2(options) {
	var url = _.get(options, "url", ""),
		bearer = _.get(options, "bearer_token_endpoint", "");

	return new auth.OAuth2(
		_.get(options, "consumer_key", ""),
		_.get(options, "consumer_secret", ""),
		url,
		null,
		bearer.replace(url, ""),
		null);
}

// OAuthEcho(realm, verify_credentials, consumerKey, consumerSecret, version, signatureMethod, nonceSize, customHeaders)
function echo(params) {
	return new auth.OAuthEcho(
		_.get(options, "realm", ""),
		_.get(options, "verify_credentials", ""),
		_.get(options, "consumer_key", ""),
		_.get(options, "consumer_secret", ""),
		_.get(options, "oauth_version", "1.0A"),
		_.get(options, "sig_method", "HMAC-SHA1"),
		null,
		null);
}