/**
 * NINJS-CORE Plugins
 * Standard Web/Html Plugin wrappers
 */

var _ = require("lodash");

/**
 * GitHub Plugins
 */

var github = {

	defaults: {
		base: "//lodash.com/",
		type: "watch",
		user: "",
		repo: "",
		count: true
	},

	templates: {
		watch: {
			"html": _.template('<iframe frameborder="0" src="<%=src%>" scrolling="no" allowtransparency="false" ' +
				'style="width:105px;height:25px"></iframe>'),
			"src": _.template('<%=base%>github-btn.html?user=<%=user%>&repo=<%=repo%>&count=<%=count%>&type=<%=type%>')
		},
		fork: {
			"html": _.template('<iframe frameborder="0" src="<%=src%>" scrolling="no" allowtransparency="false" ' +
				'style="width:105px;height:25px"></iframe>'),
			"src": _.template('<%=base%>/github-btn.html?user=<%=user%>&repo=<%=repo%>&count=<%=count%>&type=<%=type%>')
		}
	},

	// create(params)
	//
	// returns the github plugin html
	//
	// github plugin factory -> defaults to "watch"
	//
	// params
	// key   - plugin platform ("github")
	// base  - plugin platform base url ("//lodash.com/")
	// type  - plugin type ("watch", "fork")
	// user  - github user name ("lodash")
	// repo  - github user repo name ("lodash")
	// count - show count (true)
	create: function (params) {
		return github.watch(params);
	},

	// watch(params)
	//
	// returns the github watch button plugin html
	//
	// displays how many times the repo has been starred/watched
	// the current user can star/watch the repo
	watch: function (params) {
		return getMarkup(_.merge({ key: "github", type: "watch" }, params));
	},

	// fork(params)
	//
	// returns the github fork button plugin html
	//
	// displays how many times the repo has been forked
	// the current user can fork/unfork the repo
	fork: function (params) {
		return getMarkup(_.merge({ key: "github", type: "fork" }, params));
	}
};

/**
 * Twitter Plugins
 */

var twitter = {

	defaults: {
		base: "//platform.twitter.com/widgets/",
		type: "follow_button",
		dnt: true,
		screen_name: "",
		counturl: "",
		show_count: false,
		size: "m",
		url: "",
		text: ""
	},

	templates: {
		follow_button: {
			"html": _.template('<iframe frameborder="0" src="<%=src%>" scrolling="no" ' +
				'allowtransparency="false" style="width:120px;height:25px"></iframe>'),
			"src": _.template('<%=base%><%=type%>.html?dnt=<%=dnt%>&screen_name=<%=screen_name%>' +
				'&show_count=<%=show_count%>&size=<%=size%>')
		},
		tweet_button: {
			"html": _.template('<iframe frameborder="0" src="<%=src%>" scrolling="no" ' +
				'allowtransparency="false" style="width:95px;height:25px"></iframe>'),
			"src": _.template('<%=base%><%=type%>.html?dnt=<%=dnt%>&counturl=<%=screen_name%>' +
				'&show_count=<%=show_count%>&size=<%=size%>')
		}
	},

	// create(params)
	//
	// returns the twitter plugin html
	//
	// twitter plugin factory - defaults to follow_button
	//
	// params
	// key          - plugin platform ("tw")
	// base         - plugin platform base url ("//platform.twitter.com/widgets/")
	// type         - plugin type ("follow_button")
	// dnt          - dnt (true)
	// screen_name  - Twiiter user screen_name to follow/unfollow
	// show_count   - show count (false)
	// size         - button size ("s", "m", "l")
	// url          - url to tweet about ("https://lodash.com/")
	// counturl     - url to get tweet count
	// text         - text within a tweet
	create: function (params) {
		return twitter.follow_button(params);
	},

	// follow_button(params)
	//
	// returns the twitter follow button plugin html
	//
	// displays a twitter @user follow button
	// current user can follow/unfollow the screen_name
	follow_button: function (params) {
		return getMarkup(_.merge({ key: "twitter", type: "follow_button" }, params));
	},

	// tweet_button(params)
	//
	// returns the twitter tweet button plugin html
	//
	// displays a twitter tweet pop-up with populated text + " " + url
	// can also display the count of tweet for a url
	// current user can tweet the current url
	//
	// params
	// url
	// counturl
	// text
	tweet_button: function (params) {
		return getMarkup(_.merge({ key: "twitter", type: "tweet_button" }, params));
	}
};



/**
 * Exports
 */

exports = module.exports = _.assign({}, {
	getMarkup: getMarkup,
	github: github,
	twitter: twitter
});

// getMarkup(params)
// returns the plugin markup
//
// params
// key  - plugin platform ("github", "twitter")
// type - plugin type ("watch")
//      -> also specifies key[type] node for defaults/templates
//
// ... other params supplied to srcTemplate
function getMarkup(params) {
	var key = _.get(params, "key"),
		type = _.get(params, "type"),
		plat = _.get(exports, key),
		defaults = _.merge({}, _.get(plat, "defaults"), params),
		templates = _.get(plat, "templates"),
		htmlTemplate = _.template(_.get(templates, _.join([type, "html"], "."))),
		srcTemplate = _.template(_.get(templates, _.join([type, "src"], ".")));

	return htmlTemplate({ src: srcTemplate(defaults) });
}