/**
 * NINJS-CORE Request
 * Request wrapper using request module
 */

var _ = require("lodash"),
	async = require("async"),
	request = require("request"),
	_log = _.debug("web.request");

exports = module.exports = _.assign({}, request, {
	toFile: toFile,
	writeHead: writeHead,
	writeHeaders: writeHeaders,
	writeRequest: writeRequest
});

function toFile(options, callback) {
	var url = _.get(options, "url") || "",
		dest = _.get(options, "dest") || "",
		pretty = _.get(options, "pretty"),
		ex = dest ? _.path.ex(dest) || "" : "";

	pretty = _.isBoolean(pretty) ? pretty : false;

	if (!url || !_.isString(url)) return _.fail("Invalid url", callback);
	if (!dest || !_.isString(dest)) return _.fail("Invalid dest", callback);
	if (!ex) return _.fail("Invalid dest extension", callback);

	request(url, function(err, res, body) {
		if(err) return _.fail(err, callback);

		// BREAKS OCCASIONALLY WITH INVALID MUP AND IS VERY INEFFICIENT SO USE SPARINGLY
		var result = pretty && ex === "html" ? _.prettyHtml(body) : body;
		_.outputFile(dest, result, callback);
	});
}

function writeHead(options, callback) {
	var url = _.get(options, "url") || "",
		dest = _.get(options, "dest") || "";

	if (!url || !_.isString(url)) return _.fail("Invalid url", callback);
	if (!dest || !_.isString(dest)) return _.fail("Invalid dest", callback);

	async.waterfall([
		async.constant(url),
		request.head,
		async.apply(_.outputJson, dest)
	], callback);
}

function writeHeaders(options, callback) {
	var url = _.get(options, "url") || "",
		dest = _.get(options, "dest") || "";

	if (!url || !_.isString(url)) return _.fail("Invalid url", callback);
	if (!dest || !_.isString(dest)) return _.fail("Invalid dest", callback);

	async.waterfall([
		async.constant(url),
		request.head,
		async.asyncify(function(result) {
			return _.get(result.toJSON(), "headers");
		}),
		async.apply(_.outputJson, dest)
	], callback);
}

function writeRequest(options, callback) {
	var url = _.get(options, "url") || "",
		dest = _.get(options, "dest") || "";

	if (!url || !_.isString(url)) return _.fail("Invalid url", callback);
	if (!dest || !_.isString(dest)) return _.fail("Invalid dest", callback);

	async.waterfall([
		async.constant(url),
		request.head,
		async.asyncify(function (result) {
			return _.get(result.toJSON(), "request");
		}),
		async.apply(_.outputJson, dest)
	], callback);
}